# ~~~~~~~~~~~~~~~~~~~~~~~~ BEGIN PREFIX ~~~~~~~~~~~~~~~~~~~~~ #

# this code is to be added to the beginning of the file created by
# generate_tests.py. Do not use it in isolation

import unittest
import q1utils
from nltk import parse_cfg
from nltk.parse import BottomUpChartParser
from nltk.tokenize import word_tokenize
from os.path import exists

class A2Q1GeneratedUnitTests(unittest.TestCase):
    
    LEXICON_FILE_NAME = 'Lexicon'
    GRAMMAR_FILE_NAME = 'Grammar'
    PARSE_TREES_FILE_NAME = 'ParseTrees'
    
    @classmethod
    def setUpClass(cls):
        assert(hasattr(cls,'PREAMBLE'))
        
        # clear the old parse trees file. It should start with
        # the preamble specified later on in the generated section
        with open(cls.PARSE_TREES_FILE_NAME,'w') as f:
            f.write(cls.PREAMBLE)
            f.write('\n')
    
    def setUp(self):
        if not exists(self.LEXICON_FILE_NAME):
            self.skipTest("Unable to find file {} as lexicon".format(
                self.LEXICON_FILE_NAME))
        if not exists(self.GRAMMAR_FILE_NAME):
            self.skipTest("Unable to find file {} as grammar".format(
                self.GRAMMAR_FILE_NAME))
        assert exists(self.PARSE_TREES_FILE_NAME)
        
        valid,lexiconText = q1utils.sanitizeAndValidateLexicon(
            self.LEXICON_FILE_NAME)
        if not valid:
            self.skipTest("Lexicon {} is invalid.".format(
                self.LEXICON_FILE_NAME))
        
        valid,grammarText = q1utils.sanitizeAndValidateGrammar(
            self.GRAMMAR_FILE_NAME)
        if not valid:
            self.skipTest("Grammar {} is invalid.".format(
                self.GRAMMAR_FILE_NAME))
        
        allRules = grammarText + '\n' + lexiconText
        
        try:
            grammar = parse_cfg(allRules)
            self._parser = BottomUpChartParser(grammar)
        except Exception as e:
            self.skipTest(str(e))
        
    
    def _runSentenceHelper(self,sentence):
        # archetype method for parsing sentences
        trees = self._parser.nbest_parse(word_tokenize(sentence))
        with open(self.PARSE_TREES_FILE_NAME,'a') as f:
            f.write(sentence)
            f.write('\n')
            for tree in trees:
                f.write(str(tree))
                f.write('\n')
            if not trees:
                f.write('No parses')
            f.write('\n\n')
        
        return bool(trees)
    
# ~~~~~~~~~~~~~~~~~~~~~~~~ END PREFIX ~~~~~~~~~~~~~~~~~~~~~~~ #
